//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include <math.h>
#include "Uloha1_hex_t.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)

//---------------------------HEX_T::HEX_T(INT, INT, INT)---------------------
hex_t::hex_t(int x, int y, int vv)
  : vyska(vv)
{
  //nastaveni
  stred.x = x;
  stred.y = y;
  //vypocet sirky a souradnic vrcholu
  sirka = 2 * vv * sqrt(3) / 3;
  int pom1 = vv / sqrt(3) * 0.5;
  int pom2 = vv / 2;
  vrcholy[0].x = x - pom1;
  vrcholy[0].y = y - pom2;
  vrcholy[1].x = x + pom1;
  vrcholy[1].y = y - pom2;
  vrcholy[2].x = x + (pom1 * 2);
  vrcholy[2].y = y;
  vrcholy[3].x = x + pom1;
  vrcholy[3].y = y + pom2;
  vrcholy[4].x = x - pom1;
  vrcholy[4].y = y + pom2;
  vrcholy[5].x = x - (pom1 * 2);
  vrcholy[5].y = y;

  //opravne prirazeni kvuli zaokrouhujicim chybam
  //dalsi hexy by se meli tvorit z prvniho na zaklade kopirovaciho konstruktoru
  //popr. konstruktoru hex_t::hex_t(int, int, const hex_t&)
  vyska = vrcholy[4].y - vrcholy[0].y;
  sirka = vrcholy[2].x - vrcholy[5].x;

  //vlastnosti pera a stetce
  pero = new TPen;
  pero->Color = clGray;
  pero->Mode = pmCopy;
  pero->Style = psSolid;
  stetec = new TBrush;
  stetec->Color = clWhite;
  stetec->Style = bsSolid;
}

//---------------------------HEX_T::HEX_T(CONST HEX_T&)----------------------
hex_t::hex_t(const hex_t& r_hex)
  : vyska(r_hex,vyska), sirka(r_hex.sirka)
{
  pero = new TPen;
  pero->Assign(r_hex.pero);
  stetec = new TBrush;
  stetec->Assign(r_hex.stetec);
  for (int i = 0; i < 6; ++i) {
    vrcholy[i].x = r_hex.vrcholy[i].x;
    vrcholy[i].y = r_hex.vrcholy[i].y;
  }
  stred.x = r_hex.stred.x;
  stred.y = r_hex.stred.y;
}

//--------------------------~HEX_T()----------------------------------------
hex_t::~hex_t()
{
  delete pero;
  delete stetec;
}

/*
 * Tohle je druha verze specialniho konstruktoru (prvni viz nize)
 *
 *                   !!!!!!!!!!!!NEMAZAT!!!!!!!!!!!!!
 *
 *
//---------------------------HEX_T::HEX_T(INT, INT, CONST HEX_T&)------------
hex_t::hex_t(int x, int y, const hex_t& r_hex)
  //obdoba kopirovaciho konstruktoru - ale souradnice stredu a vrcholu
  //se nastavi podle vektoru posunuti (x - posun po ose x, y - posun po ose y)
  : stred(r_hex.stred.x + x, r_hex.stred.y + y)
{
  pero = new TPen;
  pero->Assign(r_hex.pero);
  stetec = new TBrush;
  stetec->Assign(r_hex.stetec);
  vrcholy[0].x = r_hex.vrcholy[0].x + x;
  vrcholy[0].y = r_hex.vrcholy[0].y + y;
  vrcholy[1].x = r_hex.vrcholy[1].x + x;
  vrcholy[1].y = r_hex.vrcholy[1].y + y;
  vrcholy[2].x = r_hex.vrcholy[2].x + x;
  vrcholy[2].y = r_hex.vrcholy[2].y + y;
  vrcholy[3].x = r_hex.vrcholy[3].x + x;
  vrcholy[3].y = r_hex.vrcholy[3].y + y;
  vrcholy[4].x = r_hex.vrcholy[4].x + x;
  vrcholy[4].y = r_hex.vrcholy[4].y + y;
  vrcholy[5].x = r_hex.vrcholy[5].x + x;
  vrcholy[5].y = r_hex.vrcholy[5].y + y;
  vrcholy[6].x = r_hex.vrcholy[6].x + x;
  vrcholy[6].y = r_hex.vrcholy[6].y + y;
}
*/

//----------------------------HEX_T::KRESLI(CONST TCANVAS&)------------------
void hex_t::kresli(TCanvas* p_c) const
{
  //nastaveni barev a vykresleni
  p_c->Pen->Assign(pero);
  p_c->Brush->Assign(stetec);
  p_c->Polygon(vrcholy, 5);
}

//-----------------------------HEX_T::JSEM_TVUJ_BOD(INT, INT)----------------
int hex_t::jsem_tvuj_bod(int x, int y) const
  //aproximace kruznici vepsanou - kupodivu dost presne pro male rozmery hexu
{
  //vzdalenost d(stred, bod) v pixelech
  int vzdalenost_bod_stred = sqrt(pow((stred.x - x), 2) + pow((stred.y - y), 2));
  //test
  if (vzdalenost_bod_stred <= vyska / 2) return 1;
  return 0;
}

//----------------------------HEX_T::HEX_T(INT, INT, CONST HEX_T&)-----------
hex_t::hex_t(int x, int y, const hex_t& r_hex_t)
{
  stred.x = x;
  stred.y = y;
  vyska = r_hex_t.vyska;
  sirka = r_hex_t.sirka;
  int pom1 = vyska / sqrt(3) * 0.5;
  int pom2 = vyska / 2;
  vrcholy[0].x = x - pom1;
  vrcholy[0].y = y - pom2;
  vrcholy[1].x = x + pom1;
  vrcholy[1].y = y - pom2;
  vrcholy[2].x = x + (pom1 * 2);
  vrcholy[2].y = y;
  vrcholy[3].x = x + pom1;
  vrcholy[3].y = y + pom2;
  vrcholy[4].x = x - pom1;
  vrcholy[4].y = y + pom2;
  vrcholy[5].x = x - (pom1 * 2);
  vrcholy[5].y = y;

  //dalsi vlastnosti hexu
  pero = new TPen;
  pero->Color = clGray;
  stetec = new TBrush;
  stetec->Color = clWhite;
}
